@@ -13,12 +13,13 @@
     #include <unistd.h>
     #include <dirent.h>
     #include <sys/stat.h>
+    #include <errno.h>
+    #include <limits.h>
     typedef int FFNativeFD;
     #define FF_INVALID_FD (-1)
     // procfs's file can be changed between read calls such as /proc/meminfo and /proc/uptime.
     // one safe way to read correct data is reading the whole file in a single read syscall
-    // 8192 comes from procps-ng: https://gitlab.com/procps-ng/procps/-/blob/master/library/meminfo.c?ref_type=heads#L39
-    #define PROC_FILE_BUFFSIZ 8192
+    #define PROC_FILE_BUFFSIZ (32 * 1024)
 #endif
 
 static inline FFNativeFD FFUnixFD2NativeFD(int unixfd)
@@ -66,9 +67,11 @@ static inline ssize_t ffReadFDData(FFNativeFD fd, size_t dataSize, void* data)
 }
 
 ssize_t ffReadFileData(const char* fileName, size_t dataSize, void* data);
+ssize_t ffReadFileDataRelative(FFNativeFD dfd, const char* fileName, size_t dataSize, void* data);
 
 bool ffAppendFDBuffer(FFNativeFD fd, FFstrbuf* buffer);
 bool ffAppendFileBuffer(const char* fileName, FFstrbuf* buffer);
+bool ffAppendFileBufferRelative(FFNativeFD dfd, const char* fileName, FFstrbuf* buffer);
 
 static inline bool ffReadFileBuffer(const char* fileName, FFstrbuf* buffer)
 {
@@ -76,12 +79,19 @@ static inline bool ffReadFileBuffer(const char* fileName, FFstrbuf* buffer)
     return ffAppendFileBuffer(fileName, buffer);
 }
 
+static inline bool ffReadFileBufferRelative(FFNativeFD dfd, const char* fileName, FFstrbuf* buffer)
+{
+    ffStrbufClear(buffer);
+    return ffAppendFileBufferRelative(dfd, fileName, buffer);
+}
+
 //Bit flags, combine with |
-typedef enum FFPathType
+typedef enum __attribute__((__packed__)) FFPathType
 {
     FF_PATHTYPE_FILE = 1 << 0,
     FF_PATHTYPE_DIRECTORY = 1 << 1,
     FF_PATHTYPE_ANY = FF_PATHTYPE_FILE | FF_PATHTYPE_DIRECTORY,
+    FF_PATHTYPE_FORCE_UNSIGNED = UINT8_MAX,
 } FFPathType;
 
 static inline bool ffPathExists(const char* path, FFPathType pathType)
@@ -211,3 +221,8 @@ static inline bool ffSearchUserConfigFile(const FFlist* configDirs, const char*
 
     return false;
 }
+
+#ifdef _WIN32
+// Only O_RDONLY is supported
+HANDLE openat(HANDLE dfd, const char* fileName, bool directory);
+#endif
