@@ -1,6 +1,7 @@
 #include "io.h"
 #include "fastfetch.h"
 #include "util/stringUtils.h"
+#include "common/time.h"
 
 #include <fcntl.h>
 #include <termios.h>
@@ -14,8 +15,11 @@
 
 #if FF_HAVE_WORDEXP
     #include <wordexp.h>
+#elif FF_HAVE_GLOB
+    #warning "<wordexp.h> is not available, use <glob.h> instead"
+    #include <glob.h>
 #else
-    #warning "<wordexp.h> not available"
+    #warning "Neither <wordexp.h> nor <glob.h> is available"
 #endif
 
 static void createSubfolders(const char* fileName)
@@ -34,7 +38,7 @@ static void createSubfolders(const char* fileName)
 bool ffWriteFileData(const char* fileName, size_t dataSize, const void* data)
 {
     int openFlagsModes = O_WRONLY | O_CREAT | O_TRUNC | O_CLOEXEC;
-    int openFlagsRights = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
+    mode_t openFlagsRights = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
 
     int FF_AUTO_CLOSE_FD fd = open(fileName, openFlagsModes, openFlagsRights);
     if(fd == -1)
@@ -105,6 +109,15 @@ ssize_t ffReadFileData(const char* fileName, size_t dataSize, void* data)
     return ffReadFDData(fd, dataSize, data);
 }
 
+ssize_t ffReadFileDataRelative(int dfd, const char* fileName, size_t dataSize, void* data)
+{
+    int FF_AUTO_CLOSE_FD fd = openat(dfd, fileName, O_RDONLY | O_CLOEXEC);
+    if(fd == -1)
+        return -1;
+
+    return ffReadFDData(fd, dataSize, data);
+}
+
 bool ffAppendFileBuffer(const char* fileName, FFstrbuf* buffer)
 {
     int FF_AUTO_CLOSE_FD fd = open(fileName, O_RDONLY | O_CLOEXEC);
@@ -114,6 +127,15 @@ bool ffAppendFileBuffer(const char* fileName, FFstrbuf* buffer)
     return ffAppendFDBuffer(fd, buffer);
 }
 
+bool ffAppendFileBufferRelative(int dfd, const char* fileName, FFstrbuf* buffer)
+{
+    int FF_AUTO_CLOSE_FD fd = openat(dfd, fileName, O_RDONLY | O_CLOEXEC);
+    if(fd == -1)
+        return false;
+
+    return ffAppendFDBuffer(fd, buffer);
+}
+
 bool ffPathExpandEnv(FF_MAYBE_UNUSED const char* in, FF_MAYBE_UNUSED FFstrbuf* out)
 {
     bool result = false;
@@ -121,17 +143,31 @@ bool ffPathExpandEnv(FF_MAYBE_UNUSED const char* in, FF_MAYBE_UNUSED FFstrbuf* o
     #if FF_HAVE_WORDEXP // https://github.com/termux/termux-packages/pull/7056
 
     wordexp_t exp;
-    if(wordexp(in, &exp, 0) != 0)
+    if (wordexp(in, &exp, 0) != 0)
         return false;
 
-    if (exp.we_wordc == 1)
+    if (exp.we_wordc >= 1)
     {
         result = true;
-        ffStrbufSetS(out, exp.we_wordv[0]);
+        ffStrbufSetS(out, exp.we_wordv[exp.we_wordc > 1 ? ffTimeGetNow() % exp.we_wordc : 0]);
     }
 
     wordfree(&exp);
 
+    #elif FF_HAVE_GLOB
+
+    glob_t gb;
+    if (glob(in, GLOB_NOSORT | GLOB_TILDE, NULL, &gb) != 0)
+        return false;
+
+    if (gb.gl_matchc >= 1)
+    {
+        result = true;
+        ffStrbufSetS(out, gb.gl_pathv[gb.gl_matchc > 1 ? ffTimeGetNow() % gb.gl_matchc : 0]);
+    }
+
+    globfree(&gb);
+
     #endif
 
     return result;
@@ -191,14 +227,24 @@ const char* ffGetTerminalResponse(const char* request, int nParams, const char*
         ssize_t nRead = read(ftty, buffer + bytesRead, sizeof(buffer) - bytesRead - 1);
 
         if (nRead <= 0)
+        {
+            va_end(args);
             return "read(STDIN_FILENO, buffer, sizeof(buffer) - 1) failed";
+        }
 
         bytesRead += (size_t) nRead;
         buffer[bytesRead] = '\0';
 
-        int ret = vsscanf(buffer, format, args);
+        va_list cargs;
+        va_copy(cargs, args);
+        int ret = vsscanf(buffer, format, cargs);
+        va_end(cargs);
+
         if (ret <= 0)
+        {
+            va_end(args);
             return "vsscanf(buffer, format, args) failed";
+        }
         if (ret >= nParams)
             break;
     }
@@ -210,6 +256,11 @@ const char* ffGetTerminalResponse(const char* request, int nParams, const char*
 
 bool ffSuppressIO(bool suppress)
 {
+    #ifndef NDEBUG
+    if (instance.config.display.debugMode)
+        return false;
+    #endif
+
     static bool init = false;
     static int origOut = -1;
     static int origErr = -1;
@@ -239,7 +290,7 @@ bool ffSuppressIO(bool suppress)
 
 void listFilesRecursively(uint32_t baseLength, FFstrbuf* folder, uint8_t indentation, const char* folderName, bool pretty)
 {
-    FF_AUTO_CLOSE_FD int dfd = open(folder->chars, O_RDONLY);
+    FF_AUTO_CLOSE_FD int dfd = open(folder->chars, O_RDONLY | O_CLOEXEC);
     if (dfd < 0)
         return;
 
@@ -264,7 +315,7 @@ void listFilesRecursively(uint32_t baseLength, FFstrbuf* folder, uint8_t indenta
             continue;
 
         bool isDir = false;
-#ifndef __sun
+#if !defined(__sun) && !defined(__HAIKU__)
         if(entry->d_type != DT_UNKNOWN && entry->d_type != DT_LNK)
             isDir = entry->d_type == DT_DIR;
         else
