@@ -37,6 +37,10 @@ static bool isPhysicalDevice(const struct mntent* device)
     if(ffStrEquals(device->mnt_type, "zfs"))
         return true;
 
+    //sshfs
+    if(ffStrEquals(device->mnt_type, "fuse.sshfs"))
+        return true;
+
     //Pseudo filesystems don't have a device in /dev
     if(!ffStrStartsWith(device->mnt_fsname, "/dev/"))
         return false;
@@ -220,7 +224,7 @@ static bool isRemovable(FFDisk* currentDisk)
 
 static void detectType(const FFlist* disks, FFDisk* currentDisk, struct mntent* device)
 {
-    if(ffStrbufStartsWithS(&currentDisk->mountpoint, "/boot") || ffStrbufStartsWithS(&currentDisk->mountpoint, "/efi"))
+    if(hasmntopt(device, "x-gvfs-hide") || hasmntopt(device, "hidden"))
         currentDisk->type = FF_DISK_VOLUME_TYPE_HIDDEN_BIT;
     else if(isSubvolume(disks, currentDisk))
         currentDisk->type = FF_DISK_VOLUME_TYPE_SUBVOLUME_BIT;
@@ -281,7 +285,7 @@ const char* ffDetectDisksImpl(FFDiskOptions* options, FFlist* disks)
     {
         if (__builtin_expect(options->folders.length, 0))
         {
-            if (!ffDiskMatchMountpoint(options, device->mnt_dir))
+            if (!ffDiskMatchMountpoint(&options->folders, device->mnt_dir))
                 continue;
         }
         else if(!isPhysicalDevice(device))
