@@ -22,9 +22,11 @@
 
 static bool getFileVersion(const FFstrbuf* exePath, const wchar_t* stringName, FFstrbuf* version)
 {
-    wchar_t exePathW[PATH_MAX];
+    wchar_t exePathW[PATH_MAX + 1];
     int len = MultiByteToWideChar(CP_UTF8, 0, exePath->chars, (int)exePath->length, exePathW, ARRAY_SIZE(exePathW));
     if (len <= 0) return false;
+    assert(len < (int) ARRAY_SIZE(exePathW));
+    exePathW[len] = L'\0';
     return ffGetFileVersion(exePathW, stringName, version);
 }
 
@@ -67,6 +69,7 @@ static bool getShellVersionBash(FFstrbuf* exe, FFstrbuf* exePath, FFstrbuf* vers
     if (*path == '\0')
         path = exe->chars;
     ffBinaryExtractStrings(path, extractBashVersion, version, (uint32_t) strlen("@(#)Bash version 0.0.0(0) release GNU"));
+    if (version->length > 0) return true;
 
     if(!getExeVersionRaw(exe, version))
         return false;
