@@ -6,12 +6,11 @@
 #include "util/stringUtils.h"
 
 #define FF_CPUUSAGE_DISPLAY_NAME "CPU Usage"
-#define FF_CPUUSAGE_NUM_FORMAT_ARGS 8
 
 void ffPrintCPUUsage(FFCPUUsageOptions* options)
 {
     FF_LIST_AUTO_DESTROY percentages = ffListCreate(sizeof(double));
-    const char* error = ffGetCpuUsageResult(&percentages);
+    const char* error = ffGetCpuUsageResult(options, &percentages);
 
     if(error)
     {
@@ -28,6 +27,12 @@ void ffPrintCPUUsage(FFCPUUsageOptions* options)
         if (*percent == *percent)
         {
             sumValue += *percent;
+
+            #if WIN32
+            // Windows may return values greater than 100%, cap them to 100%
+            if (*percent > 100) *percent = 100;
+            #endif
+
             if (*percent > maxValue)
             {
                 maxValue = *percent;
@@ -43,6 +48,12 @@ void ffPrintCPUUsage(FFCPUUsageOptions* options)
         ++index;
     }
     double avgValue = sumValue / (double) valueCount;
+    #if WIN32
+    // See above comment
+    if (avgValue > 100) avgValue = 100;
+    #endif
+
+    FFPercentageTypeFlags percentType = options->percent.type == 0 ? instance.config.display.percentType : options->percent.type;
 
     if(options->moduleArgs.outputFormat.length == 0)
     {
@@ -51,9 +62,9 @@ void ffPrintCPUUsage(FFCPUUsageOptions* options)
         FF_STRBUF_AUTO_DESTROY str = ffStrbufCreate();
         if (!options->separate)
         {
-            if(instance.config.display.percentType & FF_PERCENTAGE_TYPE_BAR_BIT)
+            if(percentType & FF_PERCENTAGE_TYPE_BAR_BIT)
                 ffPercentAppendBar(&str, avgValue, options->percent, &options->moduleArgs);
-            if(instance.config.display.percentType & FF_PERCENTAGE_TYPE_NUM_BIT)
+            if(percentType & FF_PERCENTAGE_TYPE_NUM_BIT)
             {
                 if(str.length > 0)
                     ffStrbufAppendC(&str, ' ');
@@ -74,26 +85,33 @@ void ffPrintCPUUsage(FFCPUUsageOptions* options)
     else
     {
         FF_STRBUF_AUTO_DESTROY avgNum = ffStrbufCreate();
-        ffPercentAppendNum(&avgNum, avgValue, options->percent, false, &options->moduleArgs);
+        if (percentType & FF_PERCENTAGE_TYPE_NUM_BIT)
+            ffPercentAppendNum(&avgNum, avgValue, options->percent, false, &options->moduleArgs);
         FF_STRBUF_AUTO_DESTROY avgBar = ffStrbufCreate();
-        ffPercentAppendBar(&avgBar, avgValue, options->percent, &options->moduleArgs);
+        if (percentType & FF_PERCENTAGE_TYPE_BAR_BIT)
+            ffPercentAppendBar(&avgBar, avgValue, options->percent, &options->moduleArgs);
         FF_STRBUF_AUTO_DESTROY minNum = ffStrbufCreate();
-        ffPercentAppendNum(&minNum, minValue, options->percent, false, &options->moduleArgs);
+        if (percentType & FF_PERCENTAGE_TYPE_NUM_BIT)
+            ffPercentAppendNum(&minNum, minValue, options->percent, false, &options->moduleArgs);
         FF_STRBUF_AUTO_DESTROY minBar = ffStrbufCreate();
-        ffPercentAppendBar(&minBar, minValue, options->percent, &options->moduleArgs);
+        if (percentType & FF_PERCENTAGE_TYPE_BAR_BIT)
+            ffPercentAppendBar(&minBar, minValue, options->percent, &options->moduleArgs);
         FF_STRBUF_AUTO_DESTROY maxNum = ffStrbufCreate();
-        ffPercentAppendNum(&maxNum, maxValue, options->percent, false, &options->moduleArgs);
+        if (percentType & FF_PERCENTAGE_TYPE_NUM_BIT)
+            ffPercentAppendNum(&maxNum, maxValue, options->percent, false, &options->moduleArgs);
         FF_STRBUF_AUTO_DESTROY maxBar = ffStrbufCreate();
-        ffPercentAppendBar(&maxBar, maxValue, options->percent, &options->moduleArgs);
-        FF_PRINT_FORMAT_CHECKED(FF_CPUUSAGE_DISPLAY_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, FF_CPUUSAGE_NUM_FORMAT_ARGS, ((FFformatarg[]){
-            {FF_FORMAT_ARG_TYPE_STRBUF, &avgNum, "avg"},
-            {FF_FORMAT_ARG_TYPE_STRBUF, &maxNum, "max"},
-            {FF_FORMAT_ARG_TYPE_UINT, &maxIndex, "max-index"},
-            {FF_FORMAT_ARG_TYPE_STRBUF, &minNum, "min"},
-            {FF_FORMAT_ARG_TYPE_UINT, &minIndex, "min-index"},
-            {FF_FORMAT_ARG_TYPE_STRBUF, &avgBar, "avg-bar"},
-            {FF_FORMAT_ARG_TYPE_STRBUF, &maxBar, "max-bar"},
-            {FF_FORMAT_ARG_TYPE_STRBUF, &minBar, "min-bar"},
+        if (percentType & FF_PERCENTAGE_TYPE_BAR_BIT)
+            ffPercentAppendBar(&maxBar, maxValue, options->percent, &options->moduleArgs);
+
+        FF_PRINT_FORMAT_CHECKED(FF_CPUUSAGE_DISPLAY_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, ((FFformatarg[]){
+            FF_FORMAT_ARG(avgNum, "avg"),
+            FF_FORMAT_ARG(maxNum, "max"),
+            FF_FORMAT_ARG(maxIndex, "max-index"),
+            FF_FORMAT_ARG(minNum, "min"),
+            FF_FORMAT_ARG(minIndex, "min-index"),
+            FF_FORMAT_ARG(avgBar, "avg-bar"),
+            FF_FORMAT_ARG(maxBar, "max-bar"),
+            FF_FORMAT_ARG(minBar, "min-bar"),
         }));
     }
 }
@@ -111,6 +129,12 @@ bool ffParseCPUUsageCommandOptions(FFCPUUsageOptions* options, const char* key,
         return true;
     }
 
+    if (ffStrEqualsIgnCase(subKey, "wait-time"))
+    {
+        options->waitTime = ffOptionParseUInt32(key, value);
+        return true;
+    }
+
     if (ffPercentParseCommandOptions(key, subKey, value, &options->percent))
         return true;
 
@@ -136,6 +160,12 @@ void ffParseCPUUsageJsonObject(FFCPUUsageOptions* options, yyjson_val* module)
             continue;
         }
 
+        if (ffStrEqualsIgnCase(key, "waitTime"))
+        {
+            options->waitTime = (uint32_t) yyjson_get_uint(val);
+            continue;
+        }
+
         if (ffPercentParseJsonObject(key, val, &options->percent))
             continue;
 
@@ -156,10 +186,10 @@ void ffGenerateCPUUsageJsonConfig(FFCPUUsageOptions* options, yyjson_mut_doc* do
     ffPercentGenerateJsonConfig(doc, module, defaultOptions.percent, options->percent);
 }
 
-void ffGenerateCPUUsageJsonResult(FF_MAYBE_UNUSED FFCPUUsageOptions* options, yyjson_mut_doc* doc, yyjson_mut_val* module)
+void ffGenerateCPUUsageJsonResult(FFCPUUsageOptions* options, yyjson_mut_doc* doc, yyjson_mut_val* module)
 {
     FF_LIST_AUTO_DESTROY percentages = ffListCreate(sizeof(double));
-    const char* error = ffGetCpuUsageResult(&percentages);
+    const char* error = ffGetCpuUsageResult(options, &percentages);
 
     if(error)
     {
@@ -173,36 +203,33 @@ void ffGenerateCPUUsageJsonResult(FF_MAYBE_UNUSED FFCPUUsageOptions* options, yy
     }
 }
 
-void ffPrintCPUUsageHelpFormat(void)
-{
-    FF_PRINT_MODULE_FORMAT_HELP_CHECKED(FF_CPUUSAGE_MODULE_NAME, "{1}", FF_CPUUSAGE_NUM_FORMAT_ARGS, ((const char* []) {
-        "CPU usage (percentage num, average) - avg",
-        "CPU usage (percentage num, maximum) - max",
-        "CPU core index of maximum usage - max-index",
-        "CPU usage (percentage num, minimum) - min",
-        "CPU core index of minimum usage - min-index",
-        "CPU usage (percentage bar, average) - avg-bar",
-        "CPU usage (percentage bar, maximum) - max-bar",
-        "CPU usage (percentage bar, minimum) - min-bar",
-    }));
-}
+static FFModuleBaseInfo ffModuleInfo = {
+    .name = FF_CPUUSAGE_MODULE_NAME,
+    .description = "Print CPU usage. Costs some time to collect data",
+    .parseCommandOptions = (void*) ffParseCPUUsageCommandOptions,
+    .parseJsonObject = (void*) ffParseCPUUsageJsonObject,
+    .printModule = (void*) ffPrintCPUUsage,
+    .generateJsonResult = (void*) ffGenerateCPUUsageJsonResult,
+    .generateJsonConfig = (void*) ffGenerateCPUUsageJsonConfig,
+    .formatArgs = FF_FORMAT_ARG_LIST(((FFModuleFormatArg[]) {
+        {"CPU usage (percentage num, average)", "avg"},
+        {"CPU usage (percentage num, maximum)", "max"},
+        {"CPU core index of maximum usage", "max-index"},
+        {"CPU usage (percentage num, minimum)", "min"},
+        {"CPU core index of minimum usage", "min-index"},
+        {"CPU usage (percentage bar, average)", "avg-bar"},
+        {"CPU usage (percentage bar, maximum)", "max-bar"},
+        {"CPU usage (percentage bar, minimum)", "min-bar"},
+    }))
+};
 
 void ffInitCPUUsageOptions(FFCPUUsageOptions* options)
 {
-    ffOptionInitModuleBaseInfo(
-        &options->moduleInfo,
-        FF_CPUUSAGE_MODULE_NAME,
-        "Print CPU usage. Costs some time to collect data",
-        ffParseCPUUsageCommandOptions,
-        ffParseCPUUsageJsonObject,
-        ffPrintCPUUsage,
-        ffGenerateCPUUsageJsonResult,
-        ffPrintCPUUsageHelpFormat,
-        ffGenerateCPUUsageJsonConfig
-    );
+    options->moduleInfo = ffModuleInfo;
     ffOptionInitModuleArg(&options->moduleArgs, "󰓅");
     options->separate = false;
-    options->percent = (FFColorRangeConfig) { 50, 80 };
+    options->percent = (FFPercentageModuleConfig) { 50, 80, 0 };
+    options->waitTime = 200;
 }
 
 void ffDestroyCPUUsageOptions(FFCPUUsageOptions* options)
